#include <iostream>
#include "CRC8Generator.h"
#include "CRC8GeneratorLookUp.h"
#include "CRC16Generator.h"
#include "CRC16GeneratorLookUp.h"
#include "CRC32Generator.h"
#include "CRC32GeneratorLookUp.h"

const uint8_t testData[] =  { 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39 }; //"123456789";
const uint8_t testData1[] = { 0xE4, 0x6D, 0x46, 0xF8, 0xFB, 0x16, 0xAB, 0x52, 0xED, 0x64, 0x50, 0x23, 0x63, 0x0D, 0x80, 0xBC, 0x98, 0xD8, 0x16, 0xFF, 0xEF, 0x3C, 0x4E, 0xD9, 0xBD, 0xA1, 0x85, 0x95, 0x13 };
const uint8_t testData2[] = { 0x42, 0xDF, 0x54, 0x99, 0x99, 0x20, 0x98, 0xCF, 0xF1, 0xFC, 0xF5, 0x9E, 0xED, 0x36, 0x61, 0x4A, 0xA2, 0x31, 0xF3, 0x3E, 0x02, 0xB1, 0x8A, 0x95, 0xE5, 0xF7, 0x88 };
const uint8_t testData3[] = { 0x5B, 0xB5, 0x06, 0x04, 0x52, 0xDE, 0xD8, 0xEC, 0x13, 0x35, 0x6B, 0x59, 0xC8, 0x66, 0xA4, 0xF6, 0xC9, 0xA8, 0x63, 0x03, 0xB9, 0xA8, 0xFD, 0xD9, 0x57 };

CRC8Generator crc8Generator;
CRC8GeneratorLookUp crc8GeneratorLookUp;
CRC16Generator crc16Generator;
CRC16GeneratorLookUp crc16GeneratorLookUp;
CRC32Generator crc32Generator;
CRC32GeneratorLookUp crc32GeneratorLookUp;


uint32_t stream8TestData(const uint8_t* data, size_t len) {
   while (len--)
      crc8Generator.streamCRC(*data++);
   return crc8Generator.streamFin();
}

uint32_t stream8TestDataLookUp(const uint8_t* data, size_t len) {
   while (len--) 
      crc8GeneratorLookUp.streamCRC(*data++);
   return crc8GeneratorLookUp.streamFin();
}


uint32_t stream16TestData(const uint8_t* data, size_t len) {
   while (len--)
      crc16Generator.streamCRC(*data++);
   return crc16Generator.streamFin();
}

uint32_t stream16TestDataLookUp(const uint8_t* data, size_t len) {
   while (len--)
      crc16GeneratorLookUp.streamCRCFast(*data++);
   return crc16GeneratorLookUp.streamFin();
}

uint32_t stream32TestData(const uint8_t* data, size_t len) {
   while (len--)
      crc32Generator.streamCRC(*data++);
   return crc32Generator.streamFin();
}

uint32_t stream32TestDataLookUp(const uint8_t* data, size_t len) {
   while (len--)
      crc32GeneratorLookUp.streamCRC(*data++);
   return crc32GeneratorLookUp.streamFin();
}


int main() {
    std::cout << "C++ CRC-Generator Test\n";
    
 //   std::cout << std::hex;

    std::cout << "==========================\n\n";
    std::cout << "  CRC8\n";
    std::cout << "==========================\n";

    std::cout << "   genCRC: " << (int)crc8Generator.generateCRC(testData, sizeof(testData)) << "\n";
    std::cout << "   stream: " << stream8TestData(testData, sizeof(testData)) << "\n";
    std::cout << "  genFast: " << (int)crc8GeneratorLookUp.generateCRC(testData, sizeof(testData)) << "\n";
    std::cout << "  streamF: " << stream8TestDataLookUp(testData, sizeof(testData)) << "\n";
    std::cout << "--------------------------\n";
    std::cout << "1  genCRC: " << (int)crc8Generator.generateCRC(testData1, sizeof(testData1)) << "\n";
    std::cout << "   stream: " << stream8TestData(testData1, sizeof(testData1)) << "\n";
    std::cout << "  genFast: " << (int)crc8GeneratorLookUp.generateCRC(testData1, sizeof(testData1)) << "\n";
    std::cout << "  streamF: " << stream8TestDataLookUp(testData1, sizeof(testData1)) << "\n";
    std::cout << "--------------------------\n";
    std::cout << "2  genCRC: " << (int)crc8Generator.generateCRC(testData2, sizeof(testData2)) << "\n";
    std::cout << "   stream: " << stream8TestData(testData2, sizeof(testData2)) << "\n";
    std::cout << "  genFast: " << (int)crc8GeneratorLookUp.generateCRC(testData2, sizeof(testData2)) << "\n";
    std::cout << "  streamF: " << stream8TestDataLookUp(testData2, sizeof(testData2)) << "\n";
    std::cout << "--------------------------\n";
    std::cout << "3  genCRC: " << (int)crc8Generator.generateCRC(testData3, sizeof(testData3)) << "\n";
    std::cout << "   stream: " << stream8TestData(testData3, sizeof(testData3)) << "\n";
    std::cout << "  genFast: " << (int)crc8GeneratorLookUp.generateCRC(testData3, sizeof(testData3)) << "\n";
    std::cout << "  streamF: " << stream8TestDataLookUp(testData3, sizeof(testData3)) << "\n";



    std::cout << "==========================\n\n";
    std::cout << "  CRC16\n";
    std::cout << "==========================\n";

    std::cout << "   genCRC: " << crc16Generator.generateCRC(testData, sizeof(testData)) << "\n";
    std::cout << "   stream: " << stream16TestData(testData, sizeof(testData)) << "\n";
    std::cout << "  genFast: " << crc16GeneratorLookUp.generateCRCFast(testData, sizeof(testData)) << "\n";
    std::cout << "  streamF: " << stream16TestDataLookUp(testData, sizeof(testData)) << "\n";
    std::cout << "--------------------------\n";
    std::cout << "1  genCRC: " << crc16Generator.generateCRC(testData1, sizeof(testData1)) << "\n";
    std::cout << "   stream: " << stream16TestData(testData1, sizeof(testData1)) << "\n";
    std::cout << "  genFast: " << crc16GeneratorLookUp.generateCRCFast(testData1, sizeof(testData1)) << "\n";
    std::cout << "  streamF: " << stream16TestDataLookUp(testData1, sizeof(testData1)) << "\n";
    std::cout << "--------------------------\n";
    std::cout << "2  genCRC: " << crc16Generator.generateCRC(testData2, sizeof(testData2)) << "\n";
    std::cout << "   stream: " << stream16TestData(testData2, sizeof(testData2)) << "\n";
    std::cout << "  genFast: " << crc16GeneratorLookUp.generateCRCFast(testData2, sizeof(testData2)) << "\n";
    std::cout << "  streamF: " << stream16TestDataLookUp(testData2, sizeof(testData2)) << "\n";
    std::cout << "--------------------------\n";
    std::cout << "3  genCRC: " << crc16Generator.generateCRC(testData3, sizeof(testData3)) << "\n";
    std::cout << "   stream: " << stream16TestData(testData3, sizeof(testData3)) << "\n";
    std::cout << "  genFast: " << crc16GeneratorLookUp.generateCRCFast(testData3, sizeof(testData3)) << "\n";
    std::cout << "  streamF: " << stream16TestDataLookUp(testData3, sizeof(testData3)) << "\n";


    std::cout << "==========================\n\n";
    std::cout << "  CRC32\n";
    std::cout << "==========================\n";

    std::cout << "   genCRC: " << crc32Generator.generateCRC(testData, sizeof(testData)) << "\n";
    std::cout << "   stream: " << stream32TestData(testData, sizeof(testData)) << "\n";
    std::cout << "  genFast: " << crc32GeneratorLookUp.generateCRC(testData, sizeof(testData)) << "\n";
    std::cout << "  streamF: " << stream32TestDataLookUp(testData, sizeof(testData)) << "\n";
    std::cout << "--------------------------\n";
    std::cout << "1  genCRC: " << crc32Generator.generateCRC(testData1, sizeof(testData1)) << "\n";
    std::cout << "   stream: " << stream32TestData(testData1, sizeof(testData1)) << "\n";
    std::cout << "  genFast: " << crc32GeneratorLookUp.generateCRC(testData1, sizeof(testData1)) << "\n";
    std::cout << "  streamF: " << stream32TestDataLookUp(testData1, sizeof(testData1)) << "\n";
    std::cout << "--------------------------\n";
    std::cout << "2  genCRC: " << crc32Generator.generateCRC(testData2, sizeof(testData2)) << "\n";
    std::cout << "   stream: " << stream32TestData(testData2, sizeof(testData2)) << "\n";
    std::cout << "  genFast: " << crc32GeneratorLookUp.generateCRC(testData2, sizeof(testData2)) << "\n";
    std::cout << "  streamF: " << stream32TestDataLookUp(testData2, sizeof(testData2)) << "\n";
    std::cout << "--------------------------\n";
    std::cout << "3  genCRC: " << crc32Generator.generateCRC(testData3, sizeof(testData3)) << "\n";
    std::cout << "   stream: " << stream32TestData(testData3, sizeof(testData3)) << "\n";
    std::cout << "  genFast: " << crc32GeneratorLookUp.generateCRC(testData3, sizeof(testData3)) << "\n";
    std::cout << "  streamF: " << stream32TestDataLookUp(testData3, sizeof(testData3)) << "\n";
   
}

/* Output ist:

C++ CRC-Generator Test
==========================

  CRC8
==========================
   genCRC: 244
   stream: 244
  genFast: 244
  streamF: 244
--------------------------
1  genCRC: 133
   stream: 133
  genFast: 133
  streamF: 133
--------------------------
2  genCRC: 233
   stream: 233
  genFast: 233
  streamF: 233
--------------------------
3  genCRC: 229
   stream: 229
  genFast: 229
  streamF: 229
==========================

  CRC16
==========================
   genCRC: 58828
   stream: 58828
  genFast: 58828
  streamF: 58828
--------------------------
1  genCRC: 39760
   stream: 39760
  genFast: 39760
  streamF: 39760
--------------------------
2  genCRC: 59077
   stream: 59077
  genFast: 59077
  streamF: 59077
--------------------------
3  genCRC: 32278
   stream: 32278
  genFast: 32278
  streamF: 32278
==========================

  CRC32
==========================
   genCRC: 3421780262
   stream: 3421780262
  genFast: 3421780262
  streamF: 3421780262
--------------------------
1  genCRC: 1628163719
   stream: 1628163719
  genFast: 1628163719
  streamF: 1628163719
--------------------------
2  genCRC: 2942970280
   stream: 2942970280
  genFast: 2942970280
  streamF: 2942970280
--------------------------
3  genCRC: 948803831
   stream: 948803831
  genFast: 948803831
  streamF: 948803831

*/